/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.field;

import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.field.model.FieldProvider;
import nl.cloudfarming.client.field.model.FieldService;
import nl.cloudfarming.client.field.type.FieldDataObject;
import nl.cloudfarming.client.field.type.FieldManagerImpl;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectInformation;
import org.netbeans.spi.project.ActionProvider;
import org.netbeans.spi.project.CopyOperationImplementation;
import org.netbeans.spi.project.DeleteOperationImplementation;
import org.netbeans.spi.project.ProjectState;
import org.netbeans.spi.project.ui.ProjectOpenedHook;
import org.netbeans.spi.project.ui.support.DefaultProjectOperations;
import org.openide.filesystems.*;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Timon Veenstra
 */
class FieldProject implements Project, FieldProvider, FileChangeListener {

    private final FileObject projectDir;
    private final ProjectState state;
    private Lookup lkp;
    private List<Field> fields;

    public FieldProject(FileObject dir, ProjectState state) {
        this.projectDir = dir;
        this.state = state;
    }

    @Override
    public FileObject getProjectDirectory() {
        return projectDir;
    }

    @Override
    public Lookup getLookup() {
        if (lkp == null) {
            lkp = Lookups.fixed(
                    this,
                    state, //allow outside code to mark the project as needing saving
                    new ActionProviderImpl(), //Provides standard actions like Build and Clean
                    new FieldProjectOpenHook(this),
                    new FieldProjectDeleteOperation(),
                    new FieldProjectCopyOperation(this),
                    new FieldProjectInfo(), //Project information implementation
                    new FieldManagerImpl(), // Manager
                    new FieldProjectLogicalView(this) //Logical view of project implementation
                    );
        }
        return lkp;
    }

    FileObject getFieldsFolder(boolean create) {
        FileObject result =
                projectDir.getFileObject(FieldProjectFactory.FIELDS_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(FieldProjectFactory.FIELDS_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }

    /**
     * Gets the import folder for fields
     *
     * @param create when true folder is created if not found
     * @return the found folder or in case of create false and not found null
     */
    FileObject getImportFolder(boolean create) {
        FileObject result =
                projectDir.getFileObject(FieldProjectFactory.IMPORT_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(FieldProjectFactory.IMPORT_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }

    FileObject getHelpLinesFolder(boolean create) {
        FileObject result =
                projectDir.getFileObject(FieldProjectFactory.HELP_LINES_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(FieldProjectFactory.HELP_LINES_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }

    /**
     * Read and return all fields from this projects dir
     *
     * @return List of fields
     */
    @Override
    public List<Field> getFields() {
        if (fields == null) {
            FileObject fieldsDir = getProjectDirectory().getFileObject(FieldProjectFactory.FIELDS_DIR);
            fieldsDir.addFileChangeListener(this);
            FieldManager fieldManager = Lookup.getDefault().lookup(FieldManager.class);

            assert (fieldManager != null);

            fields = new ArrayList<>();

            for (FileObject fieldFile : fieldsDir.getChildren()) {
                fields.add(fieldManager.load(fieldFile));
            }
        }
        return fields;
    }

    @Override
    public String toString() {
        return projectDir.getName();
    }

    @Override
    public void fileFolderCreated(FileEvent fe) {
        if(!fe.getFile().isFolder()){
            try {
                DataObject newDataObject = DataObject.find(fe.getFile());
                if(newDataObject instanceof FieldDataObject){
                    Field field = newDataObject.getLookup().lookup(Field.class);
                    fields.add(field);
                }
            } catch (DataObjectNotFoundException ex) {
                //Do nothing could happen. Maybe a folder is created
            }
        }
    }

    @Override
    public void fileDataCreated(FileEvent fe) {
    }

    @Override
    public void fileChanged(FileEvent fe) {
    }

    @Override
    public void fileDeleted(FileEvent fe) {
        if(!fe.getFile().isFolder()){
            try {
                DataObject newDataObject = DataObject.find(fe.getFile());
                if(newDataObject instanceof FieldDataObject){
                    Field field = newDataObject.getLookup().lookup(Field.class);
                    fields.remove(field);
                }
            } catch (DataObjectNotFoundException ex) {
                //Do nothing could happen. Maybe a folder is created
            }
        }
    }

    @Override
    public void fileRenamed(FileRenameEvent fe) {
    }

    @Override
    public void fileAttributeChanged(FileAttributeEvent fe) {
    }

    private final class ActionProviderImpl implements ActionProvider {

        private String[] supported = new String[]{
            ActionProvider.COMMAND_DELETE,
            ActionProvider.COMMAND_COPY,};

        @Override
        public String[] getSupportedActions() {
            return supported;
        }

        @Override
        public void invokeAction(String string, Lookup lookup) throws IllegalArgumentException {
            if (string.equalsIgnoreCase(ActionProvider.COMMAND_DELETE)) {
                DefaultProjectOperations.performDefaultDeleteOperation(FieldProject.this);
            }
            if (string.equalsIgnoreCase(ActionProvider.COMMAND_COPY)) {
                DefaultProjectOperations.performDefaultCopyOperation(FieldProject.this);
            }
        }

        @Override
        public boolean isActionEnabled(String command, Lookup lookup) throws IllegalArgumentException {
            if ((command.equals(ActionProvider.COMMAND_DELETE))) {
                return true;
            } else if ((command.equals(ActionProvider.COMMAND_COPY))) {
                return true;
            } else {
                throw new IllegalArgumentException(command);
            }
        }
    }

    private final class FieldProjectDeleteOperation implements DeleteOperationImplementation {

        @Override
        public void notifyDeleting() throws IOException {
        }

        @Override
        public void notifyDeleted() throws IOException {
        }

        @Override
        public List<FileObject> getMetadataFiles() {
            List<FileObject> dataFiles = new ArrayList<>();
            return dataFiles;
        }

        @Override
        public List<FileObject> getDataFiles() {
            List<FileObject> dataFiles = new ArrayList<>();
            dataFiles.add(projectDir);
            return dataFiles;
        }
    }

    private final class FieldProjectCopyOperation implements CopyOperationImplementation {

        private final FieldProject project;
        private final FileObject projectDir;

        public FieldProjectCopyOperation(FieldProject project) {
            this.project = project;
            this.projectDir = project.getProjectDirectory();
        }

        @Override
        public List<FileObject> getMetadataFiles() {
            return Collections.EMPTY_LIST;
        }

        @Override
        public List<FileObject> getDataFiles() {
            return Collections.EMPTY_LIST;
        }

        @Override
        public void notifyCopying() throws IOException {
        }

        @Override
        public void notifyCopied(Project arg0, File arg1, String arg2) throws IOException {
        }
    }

    private final class FieldProjectInfo implements ProjectInformation {

        @Override
        public Icon getIcon() {
            return new ImageIcon(ImageUtilities.loadImage(
                    "nl/cloudfarming/client/field/map.png"));
        }

        @Override
        public String getName() {
            return getProjectDirectory().getName();
        }

        @Override
        public String getDisplayName() {
            return getName();
        }

        @Override
        public void addPropertyChangeListener(PropertyChangeListener pcl) {
            //do nothing, won't change
        }

        @Override
        public void removePropertyChangeListener(PropertyChangeListener pcl) {
            //do nothing, won't change
        }

        @Override
        public Project getProject() {
            return FieldProject.this;
        }
    }

    private final class FieldProjectOpenHook extends ProjectOpenedHook {

        private FieldService fieldService;
        private FieldProject project;

        public FieldProjectOpenHook(FieldProject project) {
            this.project = project;
            fieldService = Lookup.getDefault().lookup(FieldService.class);
        }

        @Override
        protected void projectOpened() {
            fieldService.register(project);
        }

        @Override
        protected void projectClosed() {
            fieldService.unregister(project);
        }
    }
}
